

fzero = function [ITMAX = 100, private iterations = 0] args -> x

     if #args == 2
          f = args # 1;
          xinit = args # 2;
          tol = 1e-5;
     elseif #args == 3
          f = args # 1;
          xinit = args # 2;
          tol = args # 3;
     else
          panic("fzero: two or three arguments expected");
     end

     if xinit.length == 2
          a = xinit[1];     
          b = xinit[2];     
          c = b;
          fa = f(a);
          fb = f(b);
          
          if (fa > 0 && fb > 0) || (fb > 0 && fb < 0)
               panic("fzero: initial interval not bracketing root");
          end
     elseif xinit.length == 1
          if f(xinit) == 0
               x = xinit;
               return;
          end

          a = b = xinit;
          if a == 0
             dx = 0.02;
          else
             dx = a * 0.02;
          end

          root2 = sqrt(2);
          
          fb = f(b);

          do
              dx *= root2;
              a -= dx;

              if a == -inf
                   panic("search for sign change of function fails");
              end

              fa = f(a);

              if (fa > 0) != (fb > 0)
                   break;
              end

              b += dx;
              fb = f(b);

          while (fa > 0) == (fb > 0);

     else
          panic("fzero: initial value must be a real number or a vector of two real numbers");
     end

     fc = fb;

     for k = 1 : ITMAX
         if (fb > 0 && fc > 0) || (fb < 0 && fc < 0)
              c = a;
              fc = fa;
              e = d = b - a;
         end
         if abs(fc) < abs(fb)
              a = b;
              b = c;
              c = a;
              fa = fb;
              fb = fc;
              fc = fa;
         end
         tol1 = 2 * eps * abs(b) + 0.5 * tol;
         xm = 0.5 * (c-b);
         if abs(xm) <= tol1 || fb == 0.0
              x = b;
              this.iterations = k;
              return;
         end
         if abs(e) >= tol1 && abs(fa) > abs(fb)
              s = fb / fa;
              if a == c
                 p = 2 * xm * s;
                 q = 1 - s;
              else
                 q = fa / fc;
                 r = fb / fc;
                 p = s * (2 * xm * q * (q - r) - (b-a) * (r-1));
                 q = (q-1) * (r-1)*(s-1);
              end

              if p > 0
                 q = -q;
              end
              p = abs(p);
              min1 = 3 * xm * q - abs(tol1 * q);
              min2 = abs(e*q);
              if 2*p < min(min1, min2)
                   e = d;
                   d = p/q;
              else
                   d = xm;
                   e = d;
              end
          else
              d = xm;
              e = d;
          end
          a = b;
          fa = fb;
          if abs(d) > tol1
             b += d;
          else 
             if xm >= 0
                  b += abs(tol1);
             else
                  b -= abs(tol1);
             end
          end
          fb = f(b);

     end
     this.iterations = k;
     panic("too many iterations in fzero");
end
