
with("plot.x");

fig = figure.new();

h = 10 * sin(pi/3);
x = [-5, 5, 0];
y = [-h/3, -h/3, h*2/3];

/* the big solid triangle */
fig.polygon(x, y, "solid", [0.2, 1, 0.2]);

/* recursively remove the center of triangle whose vertices are x and y */
remove_triangle_center = function (figp, x, y) -> ()
    L = norm([x[2] - x[1], y[2] - y[1]]);
    xo = x.mean;
    yo = y.mean;

    /* midpoints of the sides */
    xm = [x[1] + x[2], x[2] + x[3], x[3] + x[1]] / 2;
    ym = [y[1] + y[2], y[2] + y[3], y[3] + y[1]] / 2;

    /* [1, 1, 1] is color white -- to remove the center */
    figp>>.polygon(xm, ym, "solid", [1, 1, 1]);

    theta = pi/5;
    for k = 1 : 10
         cx = xo + L * 0.08*cos(k * theta);
         cy = yo + L * 0.08*sin(k * theta);
         
         /* draw an ellipse; x and y are center coordinates
                   0.5 and 0.4 are long and short axis
                   0 is angle between long axis and horizontal
                  "contour" is the style -- other option is "solid"
                  [1, 0, 1] is the color -- r=1, g=0, b=1
          */
         figp>>.ellipse(cx, cy, 0.04 * L, 0.03 * L, 0, "contour", [1, 0, 1]);
    end

    if L >= 1
         this(figp, [xm[1], x[2], xm[2]], [ym[1], y[2], ym[2]]);
         this(figp, [xm[2], x[3], xm[3]], [ym[2], y[3], ym[3]]);
         this(figp, [xm[3], x[1], xm[1]], [ym[3], y[1], ym[1]]);
    end
end

remove_triangle_center(>>fig, x, y);

fig.setXrange([-5, 5]);
fig.setYrange([-5, 9]);

fig.title = "Sierpinsky Gasket with flower pattern";
fig.style = "empty";
fig.save("gasket2.eps")


