
/* the lorenz differential equation */

lorenz = function [a = 10, b=28, c=8/3] (t, y) -> dydt 
         d1 = a * (y[2] - y[1]);
         d2 = b*y[1] - y[2] - y[1] * y[3];
         d3 = y[1] * y[2] - c * y[3];
         dydt = [d1; d2; d3];
end


/* solve the ivp on t=[0,50] */
(x, y) = dsolve(lorenz, [0, 50], [8; 9; 10]);


/* 2d plot */
fig = figure.new();
fig.plot{x = y[:,2], y = y[:,1], linetype = "curve", color = [0.2, 1, 0.1]};
fig.save("lorenz21.eps");


fig = figure.new();
fig.plot{x = y[:,2], y = y[:,3], linetype = "curve", color = [0.7, 0.2, 0.1]};
fig.save("lorenz23.eps");

fig = figure.new();
fig.plot{x = y[:,1], y = y[:,3], linetype = "curve", color = [0.2, 0.2, 0.7]};
fig.save("lorenz13.eps");


/* for 3d view plot */

cr = y.columnrange;

fig = figure3d.new();
fig.light_direction = [2/3, 1/3, 3/5];
fig.plot([cr=cr] (y, z) -> cr[1, 1], (y, z) -> y, (y, z) -> z, cr[:, 2], cr[:, 3], [4, 4], rand(3), rand(3));
fig.plot((x, y) -> x, (x, y) -> y, [cr=cr] (x, y) -> cr[1, 3], cr[:, 1], cr[:, 2], [4, 4], rand(3), rand(3));
fig.plot((x, z) -> x, [cr=cr] (x, z) -> cr[1, 2], (x, z) -> z, cr[:, 1], cr[:, 3], [4, 4], rand(3), rand(3));

fig.curve(y[:,1], y[:,2], y[:,3], rand(3));
fig.save("lorenz3d.eps");

