
/* based on quadtx from Cleve Moler's book "Numerical Computing with Matlab" */

global.quadtxstep = function (F, a, b, tol, fa, fc, fb) -> (Q, fcount)
      
      % Recursive subfunction used by quadtx.
      
      h = b - a; 
      c = (a + b)/2;

      fd = F((a+c)/2);
      fe = F((c+b)/2);

      Q1 = h/6 * (fa + 4*fc + fb);
      Q2 = h/12 * (fa + 4*fd + 2*fc + 4*fe + fb);

      if abs(Q2 - Q1) <= tol
         Q  = Q2 + (Q2 - Q1)/15;
         fcount = 2;
      else
         (Qa, ka) = quadtxstep(F, a, c, tol, fa, fd, fc);
         (Qb, kb) = quadtxstep(F, c, b, tol, fc, fe, fb);
         Q  = Qa + Qb;
         fcount = ka + kb + 2;
      end
end

global.quad = function [private fcount] (F, a, b, tol = 1e-6) -> Q

      %QUADTX  Evaluate definite integral numerically.
      %   Q = QUADTX(F,A,B) approximates the integral of F(x) from A to B
      %   to within a tolerance of 1.e-6.  F is a string defining a function
      %   of a single variable, an inline function, a function handle, or a
      %   symbolic expression involving a single variable.
      %
      %   Q = QUADTX(F,A,B,tol) uses the given tolerance instead of 1.e-6.
      %
      %   Arguments beyond the first four, Q = QUADTX(F,a,b,tol,p1,p2,...),
      %   are passed on to the integrand, F(x,p1,p2,..).
      %
      %   [Q,fcount] = QUADTX(F,...) also counts the number of evaluations
      %   of F(x).
      %
      %   See also QUAD, QUADL, DBLQUAD, QUADGUI.
      
      % Initialization

      c = (a + b)/2;
      fa = F(a);
      fc = F(c);
      fb = F(b);
      
      % Recursive call 

      (Q, k) = quadtxstep(F, a, b, tol, fa, fc, fb);

      this.fcount = k + 3;
end      


