

/* generate a tree of n generations */
generate_tree = function (T0, fatom, xatom, n) -> T
    T = T0;
    for j = 1 : n
         t = T;
         T = "";
         for k = 1 : t.length
              if t[k] == "F"
                T = T + fatom;
              elseif t[k] == "X"
                T = T + xatom;
              else
                T = T + t[k];
              end
         end
     end
end

/* draw the tree according to commands given in a string
   this function is recursive
   figp: pointer to the figure object
   it needs to be a pointer since the fig object is to be modified in the function
	x, y: the current position
	direction: the angle of the current direction
	angle: the angle in degree of turning when commands + and - are executed
	command: the string of commands
	kp: the pointer to the current position in the command string
 */

draw_tree = function (figp, x, y, direction, angle, command, kp, color) -> ()
             k = kp>>;   /* take the index of the fist command code */
             while k <= command.length             
                  c = command[k];
                  if c == "]"
                       kp >>= k + 1; /* let kp point to the next command code */
                       return;
                  elseif c == "["
                       kp >>= k + 1;
							  /* call itself to excute the commands up to the matching ] */
                       this(figp, x, y, direction, angle, command, kp, color);
                       k = kp>>;
                       continue;
                  elseif c == "+"
                       direction += angle/180*pi;
                  elseif c == "-"
                       direction -= angle/180*pi;
                  elseif c == "F"
                       nx = x + cos(direction);
                       ny = y + sin(direction);
                       figp>>.line([x, y], [nx, ny], *, *, *, color);
                       x = nx;
                       y = ny;
                  end
                  ++k;
             end
 end

/* ------------- bush 1 ---------------------- */

fatom = "FF-[-F+F+F]+[+F-F-F]";
xatom = "";
T0 = "F";
turning_angle = 22;

bush = generate_tree (T0, fatom, xatom, 4);

/* let command pointer point to the beging of command code sequence */
k = 1;
kp = >>k;

fig = figure.new();

/* pass a pointer to fig to draw_tree function */
draw_tree(>>fig, 0, 0, pi/2, turning_angle, bush, kp, [0.1, 1, 0.7]);
fig.style="empty";
fig.save("bush1p.eps");

/* ------------- bush 2 ---------------------- */

fatom = "FF";
xatom = "F[+X]F[-X]+X";
turning_angle=20;
T0 = "X";
bush = generate_tree (T0, fatom, xatom, 6);

k = 1;
kp = >>k;

fig = figure.new();

/* pass a pointer to fig to draw_tree function */
draw_tree(>>fig, 0, 0, pi/2, turning_angle, bush, kp, [0.5, 0.8, 0.3]);
fig.style="empty";
fig.save("bush2p.eps");

/* ------------- bush 3 ---------------------- */

fatom = "FF";
xatom = "F-[[X]+X]+F[+FX]-X";
turning_angle = 22.5;

T0 = "X";
bush = generate_tree (T0, fatom, xatom, 6);

k = 1;
kp = >>k;

fig = figure.new();

/* pass a pointer to fig to draw_tree function */
draw_tree(>>fig, 0, 0, pi/2, turning_angle, bush, kp, [0.5, 0.8, 0.2]);
fig.style="empty";
fig.save("bush3p.eps");

